#!/usr/bin/perl

use warnings;
use strict;

my @rfc = ();
my %symbols = ( "??" => { unicode => 0xfffd, name => "unused" } );
# http://tools.ietf.org/rfc/rfc1345.txt
open RFC, "<", "rfc1345.txt" or die;
while (<RFC>) {
  if (/^ (\S...)   ([0-9a-f]{4})    ([A-Z][A-Z 0-9-]+(\(.*\))?)$/) {
    my $short = $1;
    my $unicode = hex($2);
    my $name = $3;
    $short =~ s/ +$//;
    $symbols{$short} = { unicode => $unicode, name => $name };
  }
  if (/^ *&charset macintosh$/) {
    while (<RFC>) {
      next if /^ *&/;
      chomp;
      s/^ +//;
      last if /^$/;
      for (split / /) {
	my $char = $symbols{$_};
	unless ($char) {
	  print STDERR "Symbol $_ undefined\n";
	  $char = { unicode => 0xfffd, name => "undefined \"$_\"" };
	}
	push @rfc, $char;
      }
    }
  }
}
close RFC or die;
@rfc == 256 or die "Wrong rfc length";

my @uni = ( $symbols{"??"} ) x 256;
# ftp://ftp.unicode.org/Public/MAPPINGS/VENDORS/APPLE/ROMAN.TXT
open UNI, "<", "ROMAN.TXT" or die;
while (<UNI>) {
  if (/^0x([0-9A-F]{2})\t0x([0-9A-F]{4})\t# (.*)$/) {
    my $byte = hex($1);
    my $unicode = hex($2);
    my $name = $3;
    $uni[$byte] = { unicode => $unicode, name => $name };
  }
}
close UNI or die;

my @unusedr = ();
my @unusedu = ();
my @numdiff = ();
my @namediff = ();
for my $byte (32..255) {
  my $ru = $rfc[$byte]{unicode};
  my $rn = $rfc[$byte]{name};
  my $uu = $uni[$byte]{unicode};
  my $un = $uni[$byte]{name};
  if ($rn eq 'unused') {
    push @unusedr, [ $byte, $ru, $rn, $uu, $un ];
  }
  elsif ($un eq 'unused') {
    push @unusedu, [ $byte, $ru, $rn, $uu, $un ];
  }
  elsif ($ru != $uu) {
    push @numdiff, [ $byte, $ru, $rn, $uu, $un ];
  }
  elsif ($rn ne $un) {
    push @namediff, [ $byte, $ru, $rn, $uu, $un ];
  }
}

if (@unusedr) {
  print "\nUnused in RFC:\n";
  for (@unusedr) {
    printf " %02x: rfc %04x %s\n     uni %04x %s\n", @$_;
  }
}
if (@unusedu) {
  print "\nUnused in UNI:\n";
  for (@unusedu) {
    printf " %02x: rfc %04x %s\n     uni %04x %s\n", @$_;
  }
}
if (@namediff) {
  print "\nDifferent names:\n";
  for (@namediff) {
    printf " %02x: rfc %04x %s\n     uni %04x %s\n", @$_;
  }
}
if (@numdiff) {
  print "\nDifferent numbers:\n";
  for (@numdiff) {
    printf " %02x: rfc %04x %s\n     uni %04x %s\n", @$_;
  }
}
